"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ensureHexPrefix = exports.hex = exports.textToUint8Array = exports.textToHexString = exports.chunks = exports.uint8ArrayToNative = exports.tryNativeToUint8Array = exports.nativeToHexString = exports.tryNativeToHexString = exports.hexToNativeString = exports.tryHexToNativeString = exports.hexToNativeAssetString = exports.tryHexToNativeAssetString = exports.tryUint8ArrayToNative = exports.hexToUint8Array = exports.uint8ArrayToHex = exports.nativeTerraHexToDenom = exports.isHexNativeTerra = void 0;
var bytes_1 = require("@ethersproject/bytes");
var web3_js_1 = require("@solana/web3.js");
var utils_1 = require("ethers/lib/utils");
var ethers_1 = require("ethers");
var cosmos_1 = require("../cosmos");
var address_1 = require("../cosmwasm/address");
var consts_1 = require("./consts");
var terra_1 = require("../terra");
/**
 *
 * Returns true iff the hex string represents a native Terra denom.
 *
 * Native assets on terra don't have an associated smart contract address, just
 * like eth isn't an ERC-20 contract on Ethereum.
 *
 * The difference is that the EVM implementations of Portal don't support eth
 * directly, and instead require swapping to an ERC-20 wrapped eth (WETH)
 * contract first.
 *
 * The Terra implementation instead supports Terra-native denoms without
 * wrapping to CW-20 token first. As these denoms don't have an address, they
 * are encoded in the Portal payloads by the setting the first byte to 1.  This
 * encoding is safe, because the first 12 bytes of the 32-byte wormhole address
 * space are not used on Terra otherwise, as cosmos addresses are 20 bytes wide.
 */
var isHexNativeTerra = function (h) { return h.startsWith("01"); };
exports.isHexNativeTerra = isHexNativeTerra;
var isLikely20ByteTerra = function (h) {
    return h.startsWith("000000000000000000000000");
};
var nativeTerraHexToDenom = function (h) {
    return Buffer.from((0, utils_1.stripZeros)((0, exports.hexToUint8Array)(h.substr(2)))).toString("ascii");
};
exports.nativeTerraHexToDenom = nativeTerraHexToDenom;
var uint8ArrayToHex = function (a) {
    return Buffer.from(a).toString("hex");
};
exports.uint8ArrayToHex = uint8ArrayToHex;
var hexToUint8Array = function (h) {
    if (h.startsWith("0x"))
        h = h.slice(2);
    return new Uint8Array(Buffer.from(h, "hex"));
};
exports.hexToUint8Array = hexToUint8Array;
/**
 *
 * Convert an address in a wormhole's 32-byte array representation into a chain's
 * native string representation.
 *
 * @throws if address is not the right length for the given chain
 */
var tryUint8ArrayToNative = function (a, chain) {
    var chainId = (0, consts_1.coalesceChainId)(chain);
    if ((0, consts_1.isEVMChain)(chainId)) {
        return (0, utils_1.hexZeroPad)((0, utils_1.hexValue)(a), 20);
    }
    else if (chainId === consts_1.CHAIN_ID_SOLANA || chainId === consts_1.CHAIN_ID_PYTHNET) {
        return new web3_js_1.PublicKey(a).toString();
    }
    else if ((0, consts_1.isTerraChain)(chainId)) {
        var h = (0, exports.uint8ArrayToHex)(a);
        if ((0, exports.isHexNativeTerra)(h)) {
            return (0, exports.nativeTerraHexToDenom)(h);
        }
        else {
            if (chainId === consts_1.CHAIN_ID_TERRA2 && !isLikely20ByteTerra(h)) {
                // terra 2 has 32 byte addresses for contracts and 20 for wallets
                return (0, cosmos_1.humanAddress)("terra", a);
            }
            return (0, cosmos_1.humanAddress)("terra", a.slice(-20));
        }
    }
    else if (chainId === consts_1.CHAIN_ID_INJECTIVE) {
        return (0, cosmos_1.humanAddress)("inj", a.slice(-20));
    }
    else if (chainId == consts_1.CHAIN_ID_WORMCHAIN) {
        // wormchain addresses are always 20 bytes.
        return (0, cosmos_1.humanAddress)("wormhole", a.slice(-20));
    }
    else if (chainId === consts_1.CHAIN_ID_XPLA) {
        return (0, cosmos_1.humanAddress)("xpla", a.slice(-20));
    }
    else if (chainId === consts_1.CHAIN_ID_NEAR) {
        throw Error("uint8ArrayToNative: Use tryHexToNativeStringNear instead.");
    }
    else if (chainId === consts_1.CHAIN_ID_OSMOSIS) {
        throw Error("uint8ArrayToNative: Osmosis not supported yet.");
    }
    else if (chainId === consts_1.CHAIN_ID_SUI) {
        throw Error("uint8ArrayToNative: Sui not supported yet.");
    }
    else if (chainId === consts_1.CHAIN_ID_APTOS) {
        throw Error("uint8ArrayToNative: Aptos not supported yet.");
    }
    else if (chainId === consts_1.CHAIN_ID_UNSET) {
        throw Error("uint8ArrayToNative: Chain id unset");
    }
    else if (chainId === consts_1.CHAIN_ID_BTC) {
        throw Error("uint8ArrayToNative: Btc not supported");
    }
    else {
        // This case is never reached
        throw Error("Don't know how to convert address for chain " + chainId);
    }
};
exports.tryUint8ArrayToNative = tryUint8ArrayToNative;
/**
 *
 * Convert an address in a wormhole's 32-byte hex representation into a chain's native
 * string representation.
 *
 * @throws if address is not the right length for the given chain
 */
var tryHexToNativeAssetString = function (h, c) {
    return (0, exports.tryHexToNativeString)(h, c);
};
exports.tryHexToNativeAssetString = tryHexToNativeAssetString;
/**
 *
 * Convert an address in a wormhole's 32-byte hex representation into a chain's native
 * string representation.
 *
 * @deprecated since 0.3.0, use [[tryHexToNativeString]] instead.
 */
var hexToNativeAssetString = function (h, c) {
    if (!h) {
        return undefined;
    }
    try {
        return (0, exports.tryHexToNativeAssetString)(h, c);
    }
    catch (e) {
        return undefined;
    }
};
exports.hexToNativeAssetString = hexToNativeAssetString;
/**
 *
 * Convert an address in a wormhole's 32-byte hex representation into a chain's native
 * string representation.
 *
 * @throws if address is not the right length for the given chain
 */
var tryHexToNativeString = function (h, c) { return (0, exports.tryUint8ArrayToNative)((0, exports.hexToUint8Array)(h), c); };
exports.tryHexToNativeString = tryHexToNativeString;
/**
 *
 * Convert an address in a wormhole's 32-byte hex representation into a chain's native
 * string representation.
 *
 * @deprecated since 0.3.0, use [[tryHexToNativeString]] instead.
 */
var hexToNativeString = function (h, c) {
    if (!h) {
        return undefined;
    }
    try {
        return (0, exports.tryHexToNativeString)(h, c);
    }
    catch (e) {
        return undefined;
    }
};
exports.hexToNativeString = hexToNativeString;
/**
 *
 * Convert an address in a chain's native representation into a 32-byte hex string
 * understood by wormhole.
 *
 * @throws if address is a malformed string for the given chain id
 */
var tryNativeToHexString = function (address, chain) {
    var chainId = (0, consts_1.coalesceChainId)(chain);
    if ((0, consts_1.isEVMChain)(chainId)) {
        return (0, exports.uint8ArrayToHex)((0, bytes_1.zeroPad)((0, bytes_1.arrayify)(address), 32));
    }
    else if (chainId === consts_1.CHAIN_ID_SOLANA || chainId === consts_1.CHAIN_ID_PYTHNET) {
        return (0, exports.uint8ArrayToHex)((0, bytes_1.zeroPad)(new web3_js_1.PublicKey(address).toBytes(), 32));
    }
    else if (chainId === consts_1.CHAIN_ID_TERRA) {
        if ((0, terra_1.isNativeDenom)(address)) {
            return ("01" +
                (0, exports.uint8ArrayToHex)((0, bytes_1.zeroPad)(new Uint8Array(Buffer.from(address, "ascii")), 31)));
        }
        else {
            return (0, exports.uint8ArrayToHex)((0, bytes_1.zeroPad)((0, cosmos_1.canonicalAddress)(address), 32));
        }
    }
    else if (chainId === consts_1.CHAIN_ID_TERRA2 ||
        chainId === consts_1.CHAIN_ID_INJECTIVE ||
        chainId === consts_1.CHAIN_ID_XPLA) {
        return (0, address_1.buildTokenId)(chainId, address);
    }
    else if (chainId == consts_1.CHAIN_ID_WORMCHAIN) {
        return (0, exports.uint8ArrayToHex)((0, bytes_1.zeroPad)((0, cosmos_1.canonicalAddress)(address), 32));
    }
    else if (chainId === consts_1.CHAIN_ID_NEAR) {
        return (0, exports.uint8ArrayToHex)((0, bytes_1.arrayify)((0, utils_1.sha256)(Buffer.from(address))));
    }
    else if (chainId === consts_1.CHAIN_ID_OSMOSIS) {
        throw Error("hexToNativeString: Osmosis not supported yet.");
    }
    else if (chainId === consts_1.CHAIN_ID_SUI) {
        throw Error("hexToNativeString: Sui not supported yet.");
    }
    else if (chainId === consts_1.CHAIN_ID_BTC) {
        throw Error("hexToNativeString: Btc not supported yet.");
    }
    else if (chainId === consts_1.CHAIN_ID_APTOS) {
        return (0, exports.uint8ArrayToHex)((0, bytes_1.zeroPad)((0, bytes_1.arrayify)(address, { allowMissingPrefix: true }), 32));
    }
    else if (chainId === consts_1.CHAIN_ID_UNSET) {
        throw Error("hexToNativeString: Chain id unset");
    }
    else {
        // If this case is reached
        throw Error("Don't know how to convert address from chain " + chainId);
    }
};
exports.tryNativeToHexString = tryNativeToHexString;
/**
 *
 * Convert an address in a chain's native representation into a 32-byte hex string
 * understood by wormhole.
 *
 * @deprecated since 0.3.0, use [[tryNativeToHexString]] instead.
 * @throws if address is a malformed string for the given chain id
 */
var nativeToHexString = function (address, chain) {
    if (!address) {
        return null;
    }
    return (0, exports.tryNativeToHexString)(address, chain);
};
exports.nativeToHexString = nativeToHexString;
/**
 *
 * Convert an address in a chain's native representation into a 32-byte array
 * understood by wormhole.
 *
 * @throws if address is a malformed string for the given chain id
 */
function tryNativeToUint8Array(address, chain) {
    var chainId = (0, consts_1.coalesceChainId)(chain);
    return (0, exports.hexToUint8Array)((0, exports.tryNativeToHexString)(address, chainId));
}
exports.tryNativeToUint8Array = tryNativeToUint8Array;
/**
 *
 * Convert an address in a chain's native representation into a 32-byte hex string
 * understood by wormhole.
 *
 * @deprecated since 0.3.0, use [[tryUint8ArrayToNative]] instead.
 * @throws if address is a malformed string for the given chain id
 */
var uint8ArrayToNative = function (a, chainId) {
    return (0, exports.hexToNativeString)((0, exports.uint8ArrayToHex)(a), chainId);
};
exports.uint8ArrayToNative = uint8ArrayToNative;
function chunks(array, size) {
    return Array.apply(0, new Array(Math.ceil(array.length / size))).map(function (_, index) { return array.slice(index * size, (index + 1) * size); });
}
exports.chunks = chunks;
function textToHexString(name) {
    return Buffer.from(name, "binary").toString("hex");
}
exports.textToHexString = textToHexString;
function textToUint8Array(name) {
    return new Uint8Array(Buffer.from(name, "binary"));
}
exports.textToUint8Array = textToUint8Array;
function hex(x) {
    return Buffer.from(ethers_1.ethers.utils.hexlify(x, { allowMissingPrefix: true }).substring(2), "hex");
}
exports.hex = hex;
function ensureHexPrefix(x) {
    return x.substring(0, 2) !== "0x" ? "0x" + x : x;
}
exports.ensureHexPrefix = ensureHexPrefix;
